@@ -29,58 +29,8 @@ our @ISA = qw( Exporter XML::LibXML::Element );
 
 our @EXPORT = qw( xforms_model );
 
-our $VERSION = "0.5.1";
+our $VERSION = "0.61";
 
-no strict "refs";
-
-## Loop through the model elements and build convience functions for them.
-foreach my $element ( keys( %XFORMS_MODEL_CHILDREN ) )
-{
-	## Create the closure ... add it to the symbol table.
-	*{ "set" . ucfirst( $element ) } = sub {
-
-		my( $self, $attribute, @data ) = @_;
-
-		## Determine of the node already exists or not.
-		my( $node ) = $self->getChildrenByTagName( $element );
-
-		## If the node is not defined, we need to create it and attach it.
-		if( !defined( $node ) )
-		{
-			$node = XML::LibXML::Element->new( $element );
-			$node->setNamespace( $XFORMS_NSURI, $XFORMS_NSPREFIX, 1 );
-			$self->appendChild( $node );
-		}
-
-		## Loop through each of the valid attributes.
-		foreach( @{ $XFORMS_MODEL_CHILDREN{ $element } } )
-		{
-			if( defined( $$attribute{ $_ } ) )
-			{
-				## Set the attribute of the node.
-				$node->setAttribute( $_, $$attribute{ $_ } );
-			}
-		}
-		
-		## Stick the data onto the node.
-		_append_array_data( $node, @data );
-	
-		return( $node );
-	};
-
-	## Function to get a child element node.
-	*{ "get" . ucfirst( $element ) } = sub {
-		
-		my $self = shift;
-
-		my( $node ) = $self->getChildrenByTagName( $element );
-
-		return( $node );
-	};
-}
-
-use strict "refs";
-				
 ##==================================================================##
 ##  Constructor(s)/Deconstructor(s)                                 ##
 ##==================================================================##
@@ -95,7 +45,8 @@ sub new
 	## Pull in what type of an object we will be.
 	my $type = shift;
 	## Pull in any arguments provided to the constructor.
-	my %params = @_;
+	my $attributes = shift;
+	my @children = @_;
 	## The object we are generating is going to be a child class of
 	## XML::LibXML's DOM objects.
 	my $self = XML::LibXML::Element->new( 'model' );
@@ -104,12 +55,10 @@ sub new
 	## Bless the class for it is good [tm].
 	bless( $self, $class );
 	## We need to set our namespace on our model element and activate it.
-	$self->setNamespace( $XFORMS_NSURI, $XFORMS_NSPREFIX, 1 );
+	$self->setNamespace( $XFORMS_NAMESPACE{xforms}, "xforms", 1 );
 	## Determine if we have an 'id' attribute and set it if we do.
-	if( defined( $params{ 'id' } ) )
-	{
-		$self->setAttribute( "id", $params{ 'id' } );
-	}
+	__xforms_attribute( $self, $attributes );
+	__xforms_children( $self, @children );
 	## Send it back to the caller all happy like.
 	return( $self );
 }
@@ -129,35 +78,267 @@ sub DESTROY
 ##  Method(s)                                                       ##
 ##==================================================================##
 
+## Loop through all of the extension elements building convience methods
+## as we go along.
+no strict 'refs';
+
+foreach my $element ( @XFORMS_MODEL_ELEMENT )
+{
+	## We need to temporarily remove the namespace prefix for our work.
+	$element =~ s/^xforms://g;
+
+##----------------------------------------------##
+##  appendCHILDENAME                            ##
+##----------------------------------------------##
+##  Method generation for the common child      ##
+##  elements of the model element.              ##
+##----------------------------------------------##
+	*{ "append" . ucfirst( $element ) } = 
+	sub {
+
+		my( $self, $attributes, @children ) = @_;
+
+		my $node = XML::LibXML::Element->new( $element );
+
+		$self->appendChild( $node );
+
+		$node->setNamespace( $XFORMS_NAMESPACE{xforms}, "xforms", 1 );
+
+		__xforms_attribute( $node, $attributes );
+		__xforms_children( $node, @children );
+
+		return( $node );
+	};
+
+##----------------------------------------------##
+##  getCHILDENAME                               ##
+##----------------------------------------------##
+##  Method generation for the retrieval of      ##
+##  common model elements.                      ##
+##----------------------------------------------##
+	*{ "get" . ucfirst( $element ) } = 
+	sub {
+
+		my $self = shift;
+	
+		my @nodes = 
+			$self->getChildrenByTagName( "xforms:$element" );
+		
+		return( @nodes );
+	};
+}
+
+use strict 'refs';
+
+##----------------------------------------------##
+##  bindControl                                 ##
+##----------------------------------------------##
+##  Method used to associate a control with a   ##
+##  model's instance data.                      ##
+##----------------------------------------------##
+sub bindControl ($$$$)
+{
+	my( $self, $control, $bind, $value ) = @_;
+
+	my( $instance ) = $self->getInstance();
+
+	if( !defined( $instance ) )
+	{
+		croak( qq|Error: Model element requires an instance child |,
+			   qq|element before controls can be bound to it.| );
+	}
+
+	## We really want the child element of the instance node.
+	$instance = $instance->firstChild;
+
+	if( ( defined( $bind ) ) && 
+		( ref( $bind ) ) && 
+		( $bind->isa( "XML::LibXML::Node" ) ) )
+	{
+		## The first case exists when we are giving a prebuilt
+		## binding node.
+		my @attributes = $bind->attributes;
+		my @children = $bind->childNodes;
+
+		my $attributes = {};
+
+		foreach( @attributes )
+		{
+			$attributes->{$_->nodeName} = $_->getValue();
+		}
+		
+		## We need to ensure we grab a copy of the nodeset and id attributes
+		## here because appendBind is destructive.
+		$bind = $attributes->{nodeset};
+		my $id = $attributes->{id};
+		
+		## We don't attach the node that was passed in, but rather
+		## we create another using it.  This allows us to do some
+		## error/consistancy checking.
+		$self->appendBind( $attributes, @children );
+		
+		## The second case is when we are just given a XPath to
+		## to the instance data.
+		$bind =~ s/^\/+//g;
+		$bind =~ s/\/+/\//g;
+		
+		## We need to set the 'bind' attribute on the node.
+		$control->setAttribute( 'bind', $id);
+		$control->removeAttribute( 'ref' );
+		
+	}
+	elsif( defined( $bind ) )
+	{
+		## The second case is when we are just given a XPath to
+		## to the instance data.
+		$bind =~ s/^\/+//g;
+		$bind =~ s/\/+/\//g;
+
+		## We need to set the 'ref' attribute on the node.
+		$control->setAttribute( 'ref', $bind );
+		$control->removeAttribute( 'bind' );
+	}
+	else
+	{
+		## The last case is the hope that the control element
+		## already contains an 'ref' element.
+		$bind = $control->getAttribute( "ref" );
+
+		if( !defined( $bind ) )
+		{
+			croak( qq|Error: A binding expression must already exist on |,
+				   qq|the control element or one must be supplied | );
+		}
+
+		## Clean the binding expression ...
+		$bind =~ s/^\/+//g;
+		$bind =~ s/\/+/\//g;
+		
+		## We will reset it to ensure that the clean version is used.
+		$control->setAttribute( 'ref', $bind );
+		$control->removeAttribute( 'bind' );
+	}			
+
+	## Break up the XPath statement into chunks.
+	my @path = split( /\//, $bind );
+
+	## Loop through each of the @path statements to ensure and build
+	## the XPath to the instance data.
+	for( my $search = 0; $search < scalar( @path ); $search++ )
+	{
+		my( $node ) = $instance->getChildrenByTagName( $path[ $search ] );
+
+		## Check to see if the node is already defined.
+		if( !defined( $node ) )
+		{
+			my $element = XML::LibXML::Element->new( $path[ $search ] );
+			$instance = $instance->appendChild( $element );
+		}
+		else
+		{
+			$instance = $node;
+		}
+	}
+	
+	## Look to see if any nodes exist under the current node.
+	if( $instance->hasChildNodes() )
+	{
+		$instance->removeChildNodes();
+	}
+
+	## Check to see if the value is text or a node.
+	if( ( ref( $value ) ) && ( $value->isa( "XML::LibXML::Node" ) ) )
+	{
+		$instance->appendChild( $value );
+	}
+	else
+	{
+		my $text = XML::LibXML::Text->new( $value );
+		$instance->appendChild( $text );
+		## The below causes a segfault.  Need to figure that one out.
+		##$instance->appendText( $value );
+	}
+	
+	## I want to make sure that we properly associate this control
+	## with the proper model.
+	$control->setAttribute( 'model', $self->getAttribute( "id" ) );
+
+	return( $instance );
+}
+
 ##----------------------------------------------##
 ##  setInstanceData                             ##
 ##----------------------------------------------##
-##  Method for assisting in the creation of     ##
-##  instance data that is not necessarily       ##
-##  associated with a control.                  ##
+##  Convience method for setting instance data  ##
+##  in the model element.                       ##
 ##----------------------------------------------##
-sub setInstanceData
+sub setInstanceData ($$$)
 {
-	my( $self, $ref, @data ) = @_;
+	my( $self, $bind, $value ) = @_;
 
-	my $instance = $self->getInstance();
+	## We make sure that $value is defined purely for the removal of
+	## perl warning messages.
+	if( !defined( $value ) )
+	{
+		$value = "";
+	}	
+
+	my( $instance ) = $self->getInstance();
 
-	## If an instance hasn't already been created, the above call
-	## will return an undefined into $instane.  We need to ensure that
-	## it is valid.
 	if( !defined( $instance ) )
 	{
-		$instance = $self->setInstance();
+		croak( qq|Error: Model element requires an instance child |,
+			   qq|element before controls can be bound to it.| );
 	}
 
-	## We need to build our instance data path if it doesn't
-	## already exist.
-	my $node = _ensure_xpath( $instance, $ref );
+	## We really want the child element of the instance node.
+	$instance = $instance->firstChild;
 
-	## Stick the data onto that node.
-	_append_array_data( $node, @data );
+	## Clean up our binding expression a bit ...
+	$bind =~ s/^\/+//g;
+	$bind =~ s/\/+/\//g;
 
-	return;
+	## Break up the XPath statement into chunks.
+	my @path = split( /\//, $bind );
+
+	## Loop through each of the @path statements to ensure and build
+	## the XPath to the instance data.
+	for( my $search = 0; $search < scalar( @path ); $search++ )
+	{
+		my( $node ) = $instance->getChildrenByTagName( $path[ $search ] );
+
+		## Check to see if the node is already defined.
+		if( !defined( $node ) )
+		{
+			my $element = XML::LibXML::Element->new( $path[ $search ] );
+			$instance = $instance->appendChild( $element );
+		}
+		else
+		{
+			$instance = $node;
+		}
+	}
+	
+	## Look to see if any nodes exist under the current node.
+	if( $instance->hasChildNodes() )
+	{
+		$instance->removeChildNodes();
+	}
+
+	## Check to see if the value is text or a node.
+	if( ( ref( $value ) ) && ( $value->isa( "XML::LibXML::Node" ) ) )
+	{
+		$instance->appendChild( $value );
+	}
+	else
+	{
+		my $text = XML::LibXML::Text->new( $value );
+		$instance->appendChild( $text );
+		## The below causes a segfault.  Need to figure that one out.
+		##$instance->appendText( $value );
+	}
+
+	return( $instance );
 }
 
 ##==================================================================##
@@ -201,7 +382,7 @@ XML::XForms::Generator::Model
 
  use XML::XForms::Generator;
 
- my $model = xforms_model( id => 'MyFirstXForms' );
+ my $model = xforms_model( { id => 'MyFirstXForms' } );
 
 =head1 DESCRIPTION
 
@@ -216,9 +397,47 @@ plus the methods listed below:
 
 =over 4 
 
-=item getAction ()
+=item appendBind ( { ATTRIBUTES }, @DATA )
 
-Returns the action child of a model.
+Sets the binding information of a model.
+This method takes a hash refernce of name => value pairs for the attributes
+of the model's child.  The attributes are attached on the basis of their
+legitamacy when compared to the XForms schema.  If it isn't a recognized
+attribute then it won't get attached.  This method also takes an array
+of XML::LibXML capable nodes and/or text data.
+
+=item appendInstance ( { ATTRIBUTES }, @DATA )
+
+Sets the instance data append of a model.
+This method takes a hash refernce of name => value pairs for the attributes
+of the model's child.  The attributes are attached on the basis of their
+legitamacy when compared to the XForms schema.  If it isn't a recognized
+attribute then it won't get attached.  This method also takes an array
+of XML::LibXML capable nodes and/or text data.
+
+=item appendExtension ( { ATTRIBUTES }, @DATA )
+
+Sets the extensions of a model.
+This method takes a hash refernce of name => value pairs for the attributes
+of the model's child.  The attributes are attached on the basis of their
+legitamacy when compared to the XForms schema.  If it isn't a recognized
+attribute then it won't get attached.  This method also takes an array
+of XML::LibXML capable nodes and/or text data.
+
+=item appendSubmission ( { ATTRIBUTES }, @DATA )
+
+Sets the submission data of a model.
+This method takes a hash refernce of name => value pairs for the attributes
+of the model's child.  The attributes are attached on the basis of their
+legitamacy when compared to the XForms schema.  If it isn't a recognized
+attribute then it won't get attached.  This method also takes an array
+of XML::LibXML capable nodes and/or text data.
+
+=item bindControl ( CONTROL, BIND, VALUE )
+
+Method that allows you to bind a control to a model's instance data.  
+This method is not necessarily needed if you use the setInstanceData
+method.
 
 =item getBind ()
 
@@ -232,80 +451,13 @@ Returns the instance data section associated with a model.
 
 Returns any extension children of a model.
 
-=item getPrivacy ()
-
-Returns the privary elements.
-
-=item getSchema ()
-
-Returns the schema child of a model.
-
-=item getSubmitInfo ()
+=item getSubmission ()
 
 Returns the submitInfo data.
 
-=item setAction ( { ATTRIBUTES }, @DATA )
+=item setInstanceData ( REF, DATA )
 
-Sets the action data of a model.
-This method takes a hash refernce of name => value pairs for the attributes
-of the model's child.  The attributes are attached on the basis of their
-legitamacy when compared to the XForms schema.  If it isn't a recognized
-attribute then it won't get attached.  This method also takes ana array
-of XML::LibXML capable nodes and/or text data.
-
-=item setBind ( { ATTRIBUTES }, @DATA )
-
-Sets the binding information of a model.
-This method takes a hash refernce of name => value pairs for the attributes
-of the model's child.  The attributes are attached on the basis of their
-legitamacy when compared to the XForms schema.  If it isn't a recognized
-attribute then it won't get attached.  This method also takes ana array
-of XML::LibXML capable nodes and/or text data.
-
-=item setInstance ( { ATTRIBUTES }, @DATA )
-
-Sets the instance data set of a model.
-This method takes a hash refernce of name => value pairs for the attributes
-of the model's child.  The attributes are attached on the basis of their
-legitamacy when compared to the XForms schema.  If it isn't a recognized
-attribute then it won't get attached.  This method also takes ana array
-of XML::LibXML capable nodes and/or text data.
-
-=item setExtension ( { ATTRIBUTES }, @DATA )
-
-Sets the extensions of a model.
-This method takes a hash refernce of name => value pairs for the attributes
-of the model's child.  The attributes are attached on the basis of their
-legitamacy when compared to the XForms schema.  If it isn't a recognized
-attribute then it won't get attached.  This method also takes ana array
-of XML::LibXML capable nodes and/or text data.
-
-=item setPrivacy ( { ATTRIBUTES }, @DATA )
-
-Sets the privacy methodology of a model.
-This method takes a hash refernce of name => value pairs for the attributes
-of the model's child.  The attributes are attached on the basis of their
-legitamacy when compared to the XForms schema.  If it isn't a recognized
-attribute then it won't get attached.  This method also takes ana array
-of XML::LibXML capable nodes and/or text data.
-
-=item setSchema ( { ATTRIBUTES }, @DATA )
-
-Sets the schema information of a model.
-This method takes a hash refernce of name => value pairs for the attributes
-of the model's child.  The attributes are attached on the basis of their
-legitamacy when compared to the XForms schema.  If it isn't a recognized
-attribute then it won't get attached.  This method also takes ana array
-of XML::LibXML capable nodes and/or text data.
-
-=item setSubmitInfo ( { ATTRIBUTES }, @DATA )
-
-Sets the submitInfo data of a model.
-This method takes a hash refernce of name => value pairs for the attributes
-of the model's child.  The attributes are attached on the basis of their
-legitamacy when compared to the XForms schema.  If it isn't a recognized
-attribute then it won't get attached.  This method also takes ana array
-of XML::LibXML capable nodes and/or text data.
+Method to set the instance data.
 
 =back
 
@@ -325,6 +477,7 @@ D. Hageman E<lt>dhageman@dracken.comE<gt>
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (c) 2002 D. Hageman (Dracken Technologies).
+
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify 
